#!/usr/bin/python
'''
    Packages required: dovecot-imapd dovecot-pop3d
'''

import subprocess, shutil, grp, os, os.path, sys, time

class Dovecot:
    def get_mailbox(self):
        return self.mailbox

    def __init__(self,user,config=None):
        '''Create test scenario.

        dovecot is configured for all protocols (imap[s] and pop3[s]), a test
        user is set up, and /var/mail/$user contains an unread and a read mail.
        '''

        self.old_version = False
        if config == None:
            if file("/etc/dovecot/dovecot.conf","r").read().find('auth_mechanisms = plain')>0:
                # Old dovecot
                config='''
protocols = imap imaps pop3 pop3s
login = imap
login = pop3
mail_extra_groups = mail

auth = auth-cram
auth_mechanisms = cram-md5
auth_passdb = passwd-file /etc/dovecot/test.passwd
auth_user = root

auth = auth-plain
auth_mechanisms = plain
auth_passdb = pam
auth_user = root

'''
                self.old_version = True
            else:
                # Modern dovecot
                config='''
protocols = imap imaps pop3 pop3s
log_timestamp = "%Y-%m-%d %H:%M:%S "
mail_extra_groups = mail
protocol imap {
}
protocol pop3 {
  pop3_uidl_format = %08Xu%08Xv
}
auth default {
  mechanisms = plain cram-md5
  passdb passwd-file {
    args = /etc/dovecot/test.passwd
  }
  passdb pam {
  }
  userdb passwd {
  }
  user = root
}
'''

        # make sure that /etc/inetd.conf exists to avoid init script errors
        self.created_inetdconf = False
        if not os.path.exists('/etc/inetd.conf'):
            open('/etc/inetd.conf', 'a')
            self.created_inetdconf = True

        # configure and restart dovecot
        if not os.path.exists('/etc/dovecot/dovecot.conf.autotest'):
            shutil.copyfile('/etc/dovecot/dovecot.conf', '/etc/dovecot/dovecot.conf.autotest')
        cfgfile = open('/etc/dovecot/dovecot.conf', 'w')
        cfgfile.write(config)
        cfgfile.close()

        file('/etc/dovecot/test.passwd','w').write('%s:{plain}%s\n' % (user.login, user.password) )

        # restart will fail if dovecot is not already running
        subprocess.call(['/etc/init.d/dovecot', 'stop'], stdout=subprocess.PIPE)
        assert subprocess.call(['/etc/init.d/dovecot', 'start'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0

        # create test mailbox with one new and one old mail
        self.mailbox = '/var/mail/' + user.login
        self.orig_mbox = \
'''From test1@test1.com Fri Nov 17 02:21:08 2006
Date: Thu, 16 Nov 2006 17:12:23 -0800
From: Test User 1 <test1@test1.com>
To: Dovecot tester <dovecot@test.com>
Subject: Test 1
Status: N

Some really important news.

From test2@test1.com Tue Nov 28 11:29:34 2006
Date: Tue, 28 Nov 2006 11:29:34 +0100
From: Test User 2 <test2@test2.com>
To: Dovecot tester <dovecot@test.com>
Subject: Test 2
Status: R

More news.

Get cracking!
'''
        open(self.mailbox, 'w').write(self.orig_mbox)
        os.chown(self.mailbox, user.uid, grp.getgrnam('mail')[2])
        os.chmod(self.mailbox, 0660)

    def __del__(self):
        # restore original configuration and restart dovecot
        os.rename('/etc/dovecot/dovecot.conf.autotest', '/etc/dovecot/dovecot.conf')
        # quiesce, default configuration has no protocols
        subprocess.call(['/etc/init.d/dovecot', 'restart'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

        if self.created_inetdconf:
            os.unlink('/etc/inetd.conf')

    def get_ssl_fingerprint(self):
        pem = '/etc/ssl/certs/dovecot.pem'
        if not os.path.exists(pem):
            pem = '/etc/ssl/certs/ssl-cert-snakeoil.pem'
        
        sp = subprocess.Popen(['openssl','x509','-in',pem,'-noout','-md5','-fingerprint'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        return sp.communicate(None)[0].split('=',1)[1].strip()
        



'''Common classes and functions for package tests.'''

import string, random, crypt, subprocess, pwd, signal, time

class TimedOutException(Exception):
    def __init__(self, value = "Timed Out"):
        self.value = value
    def __str__(self):
        return repr(self.value)

def timeout(secs, f, *args):
    def handler(signum, frame):
        raise TimedOutException()
    
    old = signal.signal(signal.SIGALRM, handler)
    result = None
    signal.alarm(secs)
    try:
        result = f(*args)
    finally:
        signal.alarm(0)
        signal.signal(signal.SIGALRM, old)

    return result

def random_string(length):
    '''Return a random string, consisting of ASCII letters, with given
    length.'''

    s = ''
    maxind = len(string.letters)-1
    for l in range(length):
        s += string.letters[random.randint(0, maxind)]
    return s

def login_exists(login):
    '''Checks whether the given login exists on the system.'''

    try:
        pwd.getpwnam(login)
        return True
    except KeyError:
        return False

def cmd(command, input = None, stderr = subprocess.STDOUT):
    '''Try to execute given command (array) and return its stdout, or return
    a textual error if it failed.'''

    try:
       sp = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=stderr, close_fds=True)
    except OSError, e:
        return [127, str(e)]

    out = sp.communicate(input)[0]
    return [sp.returncode,out]

class TestUser:
    '''Create a temporary test user and remove it again in the dtor.'''

    def __init__(self, login=None, home=True):
        '''Create a new user account with a random password. 
        
        By default, the login name is random, too, but can be explicitly
        specified with 'login'. By default, a home directory is created, this
        can be suppressed with 'home=False'.'''

        self.login = None

        if login:
            if login_exists(login):
                raise ValueError, 'login name already exists'
        else:
            while(True):
                login = random_string(8)
                if not login_exists(login):
                    break

        self.salt = random_string(2)
        self.password = random_string(8)
        self.crypted = crypt.crypt(self.password, self.salt)

        if home:
            assert subprocess.call(['useradd', '-p', self.crypted, '-m', login]) == 0
        else:
            assert subprocess.call(['useradd', '-p', self.crypted, login]) == 0

        self.login = login
        p = pwd.getpwnam(self.login)
        self.uid = p[2]
        self.gid = p[3]

    def __del__(self):
        '''Remove the created user account.'''

        if self.login:
            # seems to already have gone here
            try:
                import subprocess
            except:
                pass
            assert subprocess.call(['userdel', '-r', self.login]) == 0



#!/usr/bin/python

'''Dovecot Apport interface

Copyright (C) 2010 Canonical Ltd/
Author: Chuck Short <chuck.short@canonical.com>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
the full text of the license.
'''

from apport.hookutils import *

def add_info(report, ui):
	response = ui.yesno("The output of dovecot -n may help developers diagnose your bug more quickly, however, it may contain sensitive information. Do you want to include it in your bug report?")

	if response == None: #user canceled
		raise StopIteration

	elif response == True:
		report['DovecotConf'] = root_command_output(['/usr/sbin/dovecot', '-n'])
		

	elif response == False:
		ui.information("The contents of dovecot -n will NOT be includeded in the bug report")

	packages=['dovecot-common', 'dovecot-core', 'dovecot-dev', 'dovecot-pop3d', 'dovecot-imapd',  'mail-stack-delivery', 'dovecot-postfix']
	versions = ''
	for package in packages:
		try:
			version  = package.get_version(package)
		except:
			version = 'N/A'
		versions += '%s %s\n' %(package, version)
	report['DovecotInstalledVersions'] = versions	
             
